# API Documentation of Bash Base Library

## error messages

### error-codes

```
error-codes (bash-base)
#   1 .. unkown internal error
#   2 .. illegal number of arguments in function call
#   3 .. illegal variable name
#   4 .. illegal function arguments
#   5 .. error sourcing file
#   6 .. no such function
#   7 .. variable specified as argument undefined
#   8 .. no such file
#   9 .. no access rights
#  9x .. errors local to each function
```

### bb_error_string

```
bb_error_string <ret-vn> <error-code>
# errors: 2, 3
```

## generic tools

### bb_validate_vn

```
bb_validate_vn <caller-name> <vn> [..]
# validates one or more variable names
# valid variable names match the following regex: ^[[:alpha:]][[:alnum:]_]*$
# if caller-name is non empty a log msg with level warning will be emitted
# errors: 2, 3
```

### bb_validate_regex

```
bb_validate_regex <caller-name> <regex> <arg> [..]
# validates one or more arguments against specified regex
# if caller-name is non empty a log msg with level warning will be emitted
# errors: 2, 3
```

### bb_validate_uint

```
bb_validate_uint <caller-name> <uint> [..]
# validates if one or more arguments are unsigned integers
# if caller-name is non empty a log msg with level warning will be emitted
# errors: 2, 3
```

### bb_validate_int

```
bb_validate_int <caller-name> <int> [..]
# validates if one or more arguments are integers
# if caller-name is non empty a log msg with level warning will be emitted
# errors: 2, 3
```

## string tools

### bb_string_compare

```
bb_string_compare <string1> <string2>
# compare if two strings are equal, return 0 if they are
# errors: 2, 91 (not equal)
```

## array tools

### bb_array_contains_value

```
bb_array_contains_value <array-vn> <string>
# check if array contains value <string>, return 0 if found
# errors: 2, 91 (not found)
```

### bb_array_compare

```
bb_array_compare <src1-vn> <src2-vn>
# deep compare two arrays, return 0 if identically
# errors: 2, 3, 91 (not identically)
```

### bb_array_copy_without_vals

```
bb_array_copy_without_vals <dst-vn> <src-vn> <val> [..]
# copies array named srv-vn into array called dst-vn while skipping all
# specified vals
# errors: 2
```

### bb_array_remove_vals

```
bb_array_remove_vals <array-vn> <val> [..]
# removes specified vals from array named array-vn
# errors: 2
```

### bb_array_reverse

```
bb_array_reverse <ret-vn> <src-vn>
# returns reversed array
# same indices are used, only data is swapped (0 with n, 1 with (n-1), ..)
# erors: 2, 3
```

### bb_array_iter

```
bb_array_iter <array-vn> <iterfun-name> [<xarg> [..]]
# iter-fun must take: <array-value> <index> [<xarg> [..]]
# if target array is not empty contents will be merged in
# errors: 2, 3, 8, 91 (one or more iterfun calls failed)
```

### bb_array_map

```
bb_array_map <ret-vn> <src-vn> <mapfun-name> [<xarg> [..]]
# map-fun must take: <ret-vn> <src-value> [<xarg> [..]]
# if target array is not empty contents will be merged in
# errors: 2, 3, 8, 91 (one or more mapfun calls failed)
```

### bb_array_map2

```
bb_array_map2 <ret-vn> <src1-vn> <src2-vn> <mapfun-name> [<xarg> [..]]
# map-fun must take: <ret-vn> <src1-value> <src2-value> [<xarg> [..]]
# if target array is not empty contents will be merged in
# note: this function processes elements one by one and ignores keys
# errors: 2, 3, 8, 91 (src1 and src2 have different length),
#         92 (one or more mapfun calls failed)
```

### bb_array_fold_right

```
bb_array_fold_right <ret-vn> <fold-fun> <src-vn> <start-val> [<xarg> [..]]
# folds an array from left to right
# fold-fun must take: <ret-vn> <element> <old-result> [<xarg> [..]]
# errors: 2, 3, 8, 91 (fold-fun call failed)
```

### bb_array_fold_left

```
bb_array_fold_left <ret-vn> <fold-fun> <start-val> <src-vn> [<arg> [..]]
# folds an array from left to right
# fold-fun must take: <ret-vn> <old-result> <element> [<arg> [..]]
# errors: 2, 3, 8, 91 (fold-fun call failed)
```

### bb_array_filter

```
bb_array_filter <ret-vn> <src-vn> <filterfun-name> [<xarg> [..]]
# filterfun must take: <element> [<xarg> [..]]
# and shall return 0 if the element is to be kept in output array
# errors: 2, 3, 8
```

### bb_array_copy

```
bb_array_copy <dst-vn> <src-vn>
# copys an array, target array must exist and if non empty will be merged
# errors: same as bb_array_map
```

### bb_array_unique

```
bb_array_unique <dst-vn> <src-vn>
# copies array but skip non unique elements
# errors: 2, 3
```

### bb_array_length

```
bb_array_length <ret-vn> <array-vn>
# returns the length of an array
# errors: 2, 3
```

### bb_mapfun_identity

```
bb_mapfun_identity <dst-vn> <input-value>
# the identity fun
# errors: 2, 3
```

### bb_array_join_by

```
bb_array_join_by <ret-vn> <join-string> [<element0> [..]]
# joins array element with join-string, returns "" if no elements specified
# errors: 2, 3
```

### bb_array_of_string

```
bb_array_of_string <dst-vn> <string> [ <batch-size> ]
# converts a string into an array of strings with size batch-size and copy
# it into the array called dst-vn
# the default batch-size is 1
# note: the last batch may be shorter than batch-size
```

## hexadecimal

### bb_hex_validate

```
bb_hex_validate <caller-name> <hex-string> [..]
# validates one or more hex strings
# valid variable names match the following regex: ^[[:xdigit:]]*$
# errors: 2, 91 (illegal hex code)
```

### bb_hex_read

```
bb_hex_read <ret-vn>
# converts binary data read from stdin to its hex representation
# make sure it does not run in subshell, call it e.g.: bb_hex_read x < <(date)
# requires: /usr/bin/od
# errors: 2, 3, 91 (od not found)
```

### bb_bes_of_hex

```
bb_bes_of_hex <ret-vn> <hex>
# converts hex to backslash escape sequence, which can be converted to data
# by passing to: printf "%b"
# hex is interpreted as stream of bytes (means it must be of even length)
# errors: 2, 91 (hex is of odd length)
```

### bb_hex_write

```
bb_hex_write <hex>
# writes hex encoded data as binary data to stdout
# hex is interpreted as stream of bytes (means it must be of even length)
# errors: 2, 91 (hex is of odd length), 92 (invalid hex char)
```

### bb_hex_xor

```
bb_hex_xor <ret-vn> <hex1> <hex2>
# returns hex1 xor hex2 as hex
# errors: 2, 3, 91 (input length vary), 92 (invaid hex char)
```

## lock files

## error handling: handle internal errors in a configurable way

### bb_ierr

```
bb_ierr <msg> [..]
# call this on internal error, depending on mode it does:
#  mode "fatal": pass msg to bb_fatal
#  other mode: print error msg with specified loglevel
```

### bb_ierr_set_mode

```
bb_ierr_set_mode [fatal|<msg-loglevel>]
# default is "err"
```

### bb_fatal

```
bb_fatal <message> [..]
# end program execution after logging message prepended with 'fatal error'
# since it produces some lines of output it is meant for internal errors
# only, if you want to abort your script with a message use bb_quit
```

## msg: log/show messages

### bb_msg_levelname_of_string

```
bb_msg_levelname_of_string <ret-vn> <string>
# returns level as string if input was integer
# else returns string if it is a valid level
# errors: 2, 3, 4
```

### bb_msg_int_of_level

```
bb_msg_int_of_level
# convert level name to int or return int unmodified if in valid range
# errors: 2, 3, 4
```

### bb_msg_set_target

```
bb_msg_set_target <target-name> <type>[:<arg>] <min-level>[:<max-level]
#                   <prefix> [[no]trace]
# where type can be one of: stdout, stderr, file, syslog
# and arg is filename for file, facility for syslog
# level is one of: debug, info, notice, warning, err, crit, alert, emerg
# prefix can be: none, timestamp, prog, progpid, full
# if notrace is specified, no stack traces will be print to that target
# notrace is the default
# note: target-name must match regex ^[[:alpha:]][[:alnum:]_]*$
# errors: 2, 4
```

### bb_msg_del_target

```
bb_msg_del_target <target-name>
# errors: 2
```

### bb_msg_clear_targets

```
bb_msg_clear_targets
# remove all message targets
# errors: 2
# shellcheck disable=SC2120
```

### bb_msg

```
bb_msg <level> <string> [..]
# logs messages to configured targets
# note: will return 0, even if level is illegal
# errors: 2, 4
```

### bb_msg_from_stdin

```
bb_msg_from_stdin <level> <prefix>
# reads from fd 0 until EOF and calls bb_msg for each line read
# every line is prefixed with prefix
# will not do anything if fd 0 is connected to a terminal
# note: will stop if bb_msg fails and forward the return value
# errors: 2, 4
```

### bb_msgf

```
bb_msgf <level> <format> [<format-arg> [..]]
# logs printf style messages to configured targets
# note: will return 0, even if level is illegal
# errors: 2, 4
```

### bb_msg_stacktrace

```
bb_msg_stacktrace <ret-vn> [<skip>]
# create a human readable stacktrace, excluding this function and
# returns it as array of strings
# optional arg skip can be used to manually configure how many inner
# levels will not be displayed
# errors: 2, 3
```

## log functions (deprecated, use msg instead)

### bb_logf

```
bb_logf <message> [..]
# logs message file name in $LOG_FILE if non zero
```

### bb_log

```
bb_log <message> [..]
# logs message to stderr and if define to file name in $LOG_FILE
```

### bb_log_debug

```
bb_log_debug <level> <message> [..]
# logs message if $LOG_LEVEL is set and it is level <= $LOG_LEVEL
# errors: 2
```

## flow control and error handling

### bb_finalize

```
bb_finalize
# unregisters all traps
```

### bb_quit

```
bb_quit [<exit-code> [<log message>] [..]]
# the safe way to clean exit a script using bash_base
```

### bb_exec

```
bb_exec <cmd> [arg [..]]
# execs cmd with args if specified
```

### bb_traps_setup

```
bb_traps_setup
# will be removed in the future
```

### bb_traps_disable

```
bb_traps_disable
# will be removed in the future
```

### bb_version

```
bb_version <ret-vn>
# returns the version of bash base
# errors: 2, 3
```

### bb_register_exit_hook

```
bb_register_exit_hook [ <hook-name> ] <function-name>
# registers a function to be called on exit, it is not possible to
# pass arguments to that function
# hooks will be called sorted by hooknames
# note: the default hook-name is the function-name
```

### bb_unregister_exit_hook

```
bb_unregister_exit_hook <hookn-ame>
# removes a registered exit hook
# errors: 2
```

### bb_load_lib

```
bb_load_lib <name>
# sources file for lib specified
# if name is no absolute or local path it tries:
# /usr/libexec/<name>.bash
# /usr/libexec/<name>/<name>.bash
# errors: 2, 5, 8, 91 (lib not found)
```

### bb_load_libs

```
bb_load_libs <name> [..]
# load one or more libs but continue if some fail to load
# errors: 2, 91 (not all libs loaded)
```

### bb_require_libs

```
bb_require_libs <name> [..]
# load one or more libs and raise fatal error if any fail
# errors: 2, bb_fatal on any other error
```
